#PyFlora - Written by James Reinebold (www.reinebold.com)
#An l-system tool for rendering virtual plants.
#Based on the text Algorithmic Beauty of Plants
#See Reame.rtf for more details!

#imports
import pygame
import math
import sys
import os

from Turtle import Turtle
from Rule import RuleSystem

#constants
WIDTH = 800
HEIGHT = 600
running = True
BACKGROUND_COLOR = (33, 128, 184) #sky blue

#init pygame
pygame.init()
display = pygame.display.set_mode((WIDTH, HEIGHT), 0, 32)
clock = pygame.time.Clock()

def setupSimulation(turtles):
    """
        Create a little demo with six different types of flora.
    """
    fernRS = RuleSystem("examples" + os.sep + "fern.txt")
    vineRS = RuleSystem("examples" + os.sep + "vine.txt")
    bushRS = RuleSystem("examples" + os.sep + "bush.txt")
    treeRS = RuleSystem("examples" + os.sep + "tree.txt")
    fern2RS = RuleSystem("examples" + os.sep + "fern2.txt")
    typebRS = RuleSystem("examples" + os.sep + "typeb.txt")
    donatello = Turtle(fernRS, 150, 600, 90, 1000)
    leonardo = Turtle(vineRS, 50, 600, 90, 4000)
    turtles.append(donatello)
    turtles.append(leonardo)
    turtles.append(Turtle(treeRS, 350, 600, 90, 1000))
    turtles.append(Turtle(typebRS, 450, 600, 90, 3000))
    turtles.append(Turtle(fern2RS, 550, 600, 90, 2000))
    turtles.append(Turtle(bushRS, 650, 600, 90, 5000))


def draw(actors):
    """
        Tell the turtles to draw.
    """
    display.fill(BACKGROUND_COLOR)
    pygame.draw.line(display, [0, 100, 0], [0, 596], [800, 596], 10)
    #do all tree drawing here
    for actor in actors:
        actor.draw(display)
    pygame.display.flip()

def handleEvents():
    """
        Listen to see if the user wants to quit.
    """
    for event in pygame.event.get():
        if event.type == pygame.QUIT:
            running = False
            exit()

def update(deltaT, actors):
    """
        Update the timestep for the growth animations.
    """
    for actor in actors:
        actor.update(deltaT)


if __name__ == "__main__":
    #run script
    turtles = []
    setupSimulation(turtles)
    #gameloop
    while running:
        #FPS = 50
        elapsed = clock.tick(50)
        handleEvents()
        update(elapsed, turtles)
        draw(turtles)
            
